#include <sys/types.h>
#include <sys/ticlts.h>
#include <fcntl.h>
#include <stdio.h>
#include <tiuser.h>

/*
 *  The fd argument is a transport endpoint into the
 *  SVR4 connectionless loopback transport provider.
 */

set_user_info_opt(fd)
int fd;
{
   struct t_optmgmt *reqp;
   struct t_optmgmt *retp;
   int return_val = 0;

   struct {
       struct tcl_opt_hdr    header;
       struct tcl_opt_setid  data;
   } option_input_data, *option_return_data;

   /*
    *  Allocate space to hold the t_optmgmt structure
    *  for the request.  We set the last parameter of
    *  t_alloc() to 0.  That is because we don't want to
    *  allocate space for the opt buffer.  Instead, we will
    *  assign the opt buffer to our local
    *  ``option_input_data'' data structure
    */

   reqp = (struct t_optmgmt *)t_alloc(fd, T_OPTMGMT, 0);
   if (reqp == NULL) {
      return (-1);
   }

   /*
    *  Allocate space to hold the t_optmgmt structure
    *  for the reply.  Here, we set the last parameter of
    *  t_alloc() to T_OPT.  That is because we want
    *  to allocate space for the opt buffer of the
    *  return structure.  We could have used T_ALL as
    *  well, since the t_optmgmt structure only has one
    *  netbuf structure in it.
    */

   retp = (struct t_optmgmt *)t_alloc(fd, T_OPTMGMT, T_OPT);

   if (retp == NULL) {
      (void)t_free((char *)reqp, T_OPTMGMT);
      return (-1);
   }

   /*
    *  Fill in the ugly transport-specific data structure
    *  with the ugly transport-specific data.
    */

   option_input_data.header.hdr_thisopt_off =
                                 sizeof(struct tcl_opt_hdr);
   option_input_data.header.hdr_nexthdr_off = TCL_OPT_NOHDR;
   option_input_data.data.setid_type= TCL_OPT_SETID;
   option_input_data.data.setid_flg= TCL_IDFLG_UID;

   /* 
    *  Fill in the t_optmgmt structure that hold the options
    */

   reqp->opt.maxlen = 0;
   reqp->opt.len = sizeof (option_input_data);
   reqp->opt.buf = (char *) &option_input_data;
   reqp->flags = T_NEGOTIATE;

   if (t_optmgmt(fd, reqp, retp) == -1) {
      /*
       *  The t_optmgmt() routine failed!
       *  Set the buffer in the request options back to 0
       *  so t_free doesn't try to free it...
       */
      reqp->opt.buf = 0;
      (void)t_free((char *)reqp, T_OPTMGMT);
      (void)t_free((char *)retp, T_OPTMGMT);
      return (-1);
   }

   /*
    *  If the returned options are not the same as the
    *  options we tries to set, return an error.
    */

   if ((reqp->opt.len != retp->opt.len)
    || (memcmp(reqp->opt.buf,retp->opt.buf,reqp->opt.len))){
       return_val = -1;
   }

   /*
    *  Set the buffer in the request options back to 0 so
    *  t_free doesn't try to free it...
    */

   reqp->opt.buf = 0;
   (void)t_free((char *)reqp, T_OPTMGMT);
   (void)t_free((char *)retp, T_OPTMGMT);
   return (return_val);
}
